﻿using System.Drawing;
using GrapeCity.Documents.Pdf;
using GrapeCity.Documents.Pdf.Recognition.Structure;
using GrapeCity.Documents.Common;



class Program
{
    static void Main()
    {
        while (true)
        {
            // Display menu options
            Console.WriteLine("Choose an option:");
            Console.WriteLine("1. Find and Highlight Text in a PDF Document using C#");
            Console.WriteLine("2. Search for Text on a Specific PDF Page using C#");
            Console.WriteLine("3. Find and Highlight Text From a Specific Range of PDF Pages using C#");
            Console.WriteLine("4. Search for Text in PDF Based on Structure Tags using C#");
            Console.WriteLine("5. Search and Highlight Graphically Transformed Text in PDFs");
            Console.WriteLine("0. Exit");

            // Get user input
            Console.Write("Enter the option number: ");
            string userInput = Console.ReadLine();

            // Process user choice
            switch (userInput)
            {
                case "1":
                    Function1();
                    break;
                case "2":
                    Function2();
                    break;
                case "3":
                    Function3();
                    break;
                case "4":
                    Function4();
                    break;
                case "5":
                    Function5();
                    break;
                case "0":
                    Console.WriteLine("Exiting application. Goodbye!");
                    return;
                default:
                    Console.WriteLine("Invalid option. Please choose a valid option.");
                    break;
            }

            // Add a newline for better readability
            Console.WriteLine();
        }
    }

    // Define your functions here
    static void Function1()
    {
        Console.WriteLine("Find and Highlight Text in a PDF Document using C#");
        Console.Write("Enter word to search:");
        string searchWord = Console.ReadLine();
        // Initialize the DsPdf document instance
        var doc = new GcPdfDocument();

        using (var fs = new FileStream(Path.Combine("wetlands.pdf"), FileMode.Open, FileAccess.Read))
        {
            // Load a sample PDF  
            doc.Load(fs);
            // Use the FindText method to search text for drive, using case-insensitive, whole word match  
            var findsDrive = doc.FindText(new FindTextParams(searchWord, true, false), OutputRange.All);

            // Highlight all found text using semi-transparent orange red  
            foreach (var find in findsDrive) doc.Pages[find.PageIndex].Graphics.FillPolygon(find.Bounds[0], Color.FromArgb(100, Color.OrangeRed));

            doc.Save("1 - Search and Highlight Text.pdf");
        }
        Console.WriteLine("PDF saved");

    }
    static void Function2()
    {
        Console.WriteLine("Search for Text on a Specific PDF Page using C#");
        // Create new instance of PDF document  
        GcPdfDocument doc = new GcPdfDocument();
        using (var fs = new FileStream(Path.Combine("wetlands.pdf"), FileMode.Open, FileAccess.Read))
        {
            // Load existing PDF  
            doc.Load(fs);
            // 1. Create a new instance of FindTextParams
            var ftp = new FindTextParams("the", true, false);
            // 2. Get the text map of a page by its index, not index starts at 0 so this will search page 2  
            var tm = doc.Pages[1].GetTextMap();
            if (tm != null)
                // 3. Perform text search within the text map using FindText Method and highlight text  orange                    
                tm.FindText(ftp, (p_) =>
                {
                    doc.Pages[1].Graphics.FillPolygon(p_.Bounds[0], Color.FromArgb(100, Color.OrangeRed));
                });
            doc.Save("2 - Search Text Only Page 2.pdf");
        }
        Console.WriteLine("PDF saved");
    }
    static void Function3()
    {
        Console.WriteLine("Find and Highlight Text From a Specific Range of PDF Pages using C#");
        // Initialize the DsPdf document instance
        var doc = new GcPdfDocument();
        using (var fs = new FileStream(Path.Combine("wetlands.pdf"),FileMode.Open, FileAccess.Read))
        {
            // Load an existing document from file stream  
            doc.Load(fs);
            // Create an new FindTextParams instance  
            var ftp = new FindTextParams("the", true, false);
            // Define to and from page range properties  
            OutputRange searchRange = new OutputRange(2, 3);
            // Find all text using case-insensitive word search within the page range  
            var findsTextThe = doc.FindText(ftp, searchRange);

            foreach (var find in findsTextThe)
                doc.Pages[find.PageIndex].Graphics.FillPolygon
               (find.Bounds[0], Color.FromArgb(100, Color.OrangeRed));

            doc.Save("3 - Find and Highlight Text From a Specific Range of PDF Pages.pdf");
        }

        Console.WriteLine("PDF saved");
    }
    static void Function4()
    {
        Console.WriteLine("Search for Text in a PDF Based on Structure Tags");
        GcPdfDocument doc = new GcPdfDocument();
        using (var fs = new FileStream(Path.Combine("read-tags-to-outlines.pdf"), FileMode.Open, FileAccess.Read))
        {
            doc.Load(fs);

            // Get the LogicalStructure of the doc
            LogicalStructure ls = doc.GetLogicalStructure();
            if (ls == null || ls.Elements == null || ls.Elements.Count == 0)
            {
                // No structure tags found:
                Console.Write("No structure tags were found in the source document.", doc.Pages.Add());
                return;
            }

            // Element holds a reference of the logical structure
            Element root = ls.Elements[0];

            // Find all the H1 tags
            var find = root.Children.ToList().FindAll(e_ => e_.StructElement.Type == "H1");

            //  Loop through all found H1 tags for specific text  
            foreach (Element e in find)
            {
                var color = Color.FromArgb(64, Color.Red);
                if (e.HasContentItems)
                {
                    // Get headers text  
                    var text = e.GetText();
                    foreach (var i in e.ContentItems)
                    {

                        // Search for title with text "C1Olap"   
                        if (text.Contains("C1Olap", StringComparison.OrdinalIgnoreCase))
                        {
                            if (i is ContentItem ci)
                            {
                                var p = ci.GetParagraph();
                                if (p != null)
                                {
                                    // Get the coordinates of the found H1 tag  
                                    var rc = p.GetCoords().ToRect();
                                    rc.Offset(rc.Width, 0);
                                    // Draws highlighting around found H1  
                                    ci.Page.Graphics.DrawPolygon(p.GetCoords(), color, 1, null);
                                }
                            }
                        }
                    }
                }
                else
                    Console.WriteLine("No Text Found");
            }
            doc.Save("4 - Search for Text in a PDF Based on Structure Tags.pdf");
            Console.WriteLine("PDF saved");
        }
    }
    static void Function5()
    {
        Console.WriteLine("Find and Markup Graphically Transformed Text in PDFs");
        // Initialize the DsPdf document instance
        var doc = new GcPdfDocument();

        using (var fs = new FileStream(Path.Combine("Transformed Text.pdf"), FileMode.Open, FileAccess.Read))
        {
            // Load an existing document from file stream  
            doc.Load(fs);
            // Find all text items 'LOGO', using case-sensitive search
            var finds = doc.FindText(new FindTextParams("LOGO", false, true), OutputRange.All);
            Console.Write(finds.Count);
            // Highlight all finds: first, find all pages where the text was found  
            var pgIndices = finds.Select(f_ => f_.PageIndex).Distinct();
            // Loop through pages with found text  
            foreach (int pgIdx in pgIndices)
            {
                var page = doc.Pages[pgIdx];
                // Create a content stream of the page  
                PageContentStream pcs = page.ContentStreams.Insert(0);
                // Get the graphics included on the a pages content stream  
                var g = pcs.GetGraphics(page);
                foreach (var find in finds.Where(f_ => f_.PageIndex == pgIdx))
                {
                    foreach (var ql in find.Bounds)
                    {
                        // Set the color used to fill the polygon/highlight the found text  
                        g.FillPolygon(ql, Color.CadetBlue);
                        g.DrawPolygon(ql, Color.Blue);
                    }
                }
            }
            doc.Save("5 - Find and Markup Graphically Transformed Text in PDFs.pdf");
        }
        Console.WriteLine("PDF saved");
    }
}
